;;; negatable.lisp --- Negatable options

;; Copyright (C) 2010, 2011 Didier Verna

;; Author:        Didier Verna <didier@lrde.epita.fr>
;; Maintainer:    Didier Verna <didier@lrde.epita.fr>

;; This file is part of Clon.

;; Permission to use, copy, modify, and distribute this software for any
;; purpose with or without fee is hereby granted, provided that the above
;; copyright notice and this permission notice appear in all copies.

;; THIS SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
;; WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
;; MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
;; ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
;; WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
;; ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
;; OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.


;;; Commentary:

;; Contents management by FCM version 0.1.


;;; Code:

(in-package :com.dvlsoft.clon)
(in-readtable :com.dvlsoft.clon)


;; ==========================================================================
;; The Negatable Class
;; ==========================================================================

;; #### NOTE: currently, only the yes/no hierarchy is negatable (the negated
;; syntax meaning to return nil). The need for other kinds of negatable
;; options might pop up someday. Then, it would be advisable to design a
;; mechanism by which a + call would return something else than nil, actually
;; defined by the option class. A slot in the class below would do I think.
(defclass negatable ()
  ()
  (:documentation "The NEGATABLE Class.
This class implements the negated syntax for the switch-based hierarchy."))


;; ---------------------------
;; Help specification protocol
;; ---------------------------

(defmethod short-syntax-help-spec-prefix ((option negatable))
  #+ecl (declare (ignore option))
  "-(+)")


;; -------------------
;; Char packs protocol
;; -------------------

(defmethod negated-pack-char ((negatable negatable) &optional as-string)
  "Return NEGATABLE's negated pack character, if any."
  (potential-pack-char negatable as-string))


;;; negatable.lisp ends here
